# 项目框架简介


## lombok 简介

@NonNull : 注解在参数上, 如果该类参数为 null , 就会报出异常,  throw new NullPointException(参数名)

@Cleanup : 注释在引用变量前, 自动回收资源 默认调用 close() 方法

@Getter/@Setter : 注解在类上, 为类提供读写属性

@Getter(lazy=true) :

@ToString : 注解在类上, 为类提供 toString() 方法

@EqualsAndHashCode : 注解在类上, 为类提供 equals() 和 hashCode() 方法

@NoArgsConstructor, @RequiredArgsConstructor, @AllArgsConstructor : 注解在类上, 为类提供无参,有指定必须参数, 全参构造函数

@Data : 注解在类上, 为类提供读写属性, 此外还提供了 equals()、hashCode()、toString() 方法

@Value :

@Builder : 注解在类上, 为类提供一个内部的 Builder

@SneakThrows :

@Synchronized : 注解在方法上, 为方法提供同步锁

@Log :

@Log4j : 注解在类上, 为类提供一个属性名为 log 的 log4j 的日志对象

@Slf4j : 注解在类上, 为类提供一个属性名为 log 的 log4j 的日志对象

## SSM 简介

## springboot 简介

## springfox-swagger2简介

## logback简介

## jwt简介

## spring-security 简介

# 编码规范

| 版本  | 维护人员  | 维护时间  | 主要内容 | 
| ------------ | ------------ | ------------ | ------------ |
| v1.0  | 李丛阳  | 2018-6-06  | 基础结构搭建 |
| v2.0  | 李丛阳  | 2018-8-31  | cy.code generator v2.0.0 包含表注释 |
| v2.1  | 李丛阳  | 2018-10-31  | 增加模块配置(redis,多数据源,db log),doc注解 |

## 数据库设计规范
| 编号  | 类型 | 标题  | 描述  |
| ------------ | ------------ | ------------ | ------------ |
| 1  | 表  | 表名  | 小写，单词之间使用下划线  |
| 2  | 表  | 表名  | 中间表使用r_间隔 eg: b_r_teacher_course  |
| 3  | 表  | 表结构  | 主表必须要有6+字段(创建时间，修改时间等)，中间表不需要  |
| 4  | 列  | 列名  | 小写，单词之间使用下划线  |
| 5  | 列  | 列名  | 每一列必须要有备注  |
| 6  | 列  | 状态列  | char(1) 或 varchar(1)，备注要描述清楚状态  |


## 微服务编码规范
| 编号  | 类型 | 标题  | 描述  |
| ------------ | ------------ | ------------ | ------------ |
| 0  | 结构  | 目录规范  | 详见 projectArtifacture|
| 1  | 接口  | 提供其它服务调用的接口要标明  | 模块要单独提出一个controller,requestMappeing使用/api/ 开头|
| 2  | 接口  | 返回值  | Result / GridModel|
| 3  | 接口   |  使用restful风格 | get post pust delete，/id/ |
| 4  | 接口   |  requestMappeing命名规范 | queryUsers, queryUserByPagination, detail/businessId, delete/businessId, add, update/businessId    |
| 5  | 接口   |  swagger | controller api docs |
| 6  | 结构   |  service dao | 注释，尽量复用 |
| 7  | 结构   |  tcc | 分布式事务书写规范 |
| 8  | 结构   |  entity | @Entity @Data @NoArgsConstructor @AllArgsConstructor @Table |

## sql

~~~~
#!bin/bash
${basedir}/sql
           -- db_init.sql 
           -- db_data.sql
           -- db_clear.sql
           -- date 
                -- db.bak.sql
                -- db.alter.sql 
           
~~~~

# 模块开发

## redis

1. `application-dev.yml` 配置redis相关参数
2. `CyRedisConfigBean` 去掉 `@CyDoc(CyDocEnum.USELESS)` 开启`@ConfigurationProperties` `@Component`
3. `CyRedisConfig`  去掉 `@CyDoc(CyDocEnum.USELESS)` 开启`@Configuration`
4. `CyRedisServiceImpl`  去掉 `@CyDoc(CyDocEnum.USELESS)` 开启`@Service`

## ssl

1. 开启 application-dev.yml
~~~~
  ssl:
        key-store: classpath:client1.p12
        key-store-password: 123456
        key-store-type: PKCS12
        key-alias: client1
#并生成证书,client1.p12 放在resource下          
~~~~

## 报文加密

1. CyDecryptParamAspect.java 去掉 `@CyDoc(CyDocEnum.USELESS)`
2. application-dev.yml 配置参数 decryptKey
3. CyAopConfig 注入bean `@Bean ...`
 
## 多数据源
  
1. `application-dev.yml` 配置多数据源相关参数 参考 `power` `pmiss`
2. 配置多数据源配置类 参考 `CyDruidProPmiss` `CyDruidProPower`
3. 关闭`CyDruidConfig` 注掉`Configuration` 等, 开启  `@CyDoc(CyDocEnum.USELESS)`
4. 开启`CyMultipleDruidConfig` 和 `CyTransactionConfig` (关闭 `@CyDoc(CyDocEnum.USELESS)`, 开启`@Configuration` 等) , 并调整相关属性

## 自定义数据库日志
  
1. `logback-spring.xml` 配置数据源相关参数
2. service层 `extends CyBaseService` 使用 `this.dbInfo();` `this.dbError();`，可变参数，mess,title

