/**
 * createby : JinxLbj
 * date : 2018/10/30
 * desc :
 **/

package org.rcisoft.config;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.rcisoft.core.datasource.CyDynamicDataSource;
import org.rcisoft.core.doc.anno.CyDoc;
import org.rcisoft.core.doc.enums.CyDocEnum;
import org.rcisoft.core.druid.CyDsEnum;
import org.rcisoft.core.druid.CyDruidProPmiss;
import org.rcisoft.core.druid.CyDruidProPower;
import org.rcisoft.core.util.CyDataSourceUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/*
@Configuration // 该注解类似于spring配置文件
@EnableConfigurationProperties({CyDruidProPmiss.class,CyDruidProPower.class})
*/
@CyDoc(CyDocEnum.USELESS)
public class CyMultipleDruidConfig {

    @Value("${mybatis.mapper-locations}")
    private String localPattern;

    @Autowired
    private CyDruidProPmiss cyDruidProPmiss;

    @Autowired
    private CyDruidProPower cyDruidProPower;

    /**
     * 创建数据源(数据源的名称：方法名可以取为XXXDataSource(),XXX为数据库名称,该名称也就是数据源的名称)
     */
    @Bean
    public DataSource power() {
        DruidDataSource dataSource = CyDataSourceUtil.generateDruidByPro(cyDruidProPower);
        try {
            dataSource.init();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return dataSource;
    }

    @Bean
    public DataSource pmiss() {
        DruidDataSource dataSource = CyDataSourceUtil.generateDruidByPro(cyDruidProPmiss);
        try {
            dataSource.init();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return dataSource;
    }

    /**
     * @Primary 该注解表示在同一个接口有多个实现类可以注入的时候，默认选择哪一个，而不是让@autowire注解报错
     * @Qualifier 根据名称进行注入，通常是在具有相同的多个类型的实例的一个注入（例如有多个DataSource类型的实例）
     */
    @Bean
    @Primary
    public CyDynamicDataSource dataSource(@Qualifier("power") DataSource power,
                                          @Qualifier("pmiss") DataSource pmiss) {
        Map<Object, Object> targetDataSources = new HashMap<>();
        targetDataSources.put(CyDsEnum.POWER, power);
        targetDataSources.put(CyDsEnum.PMISS, pmiss);
        CyDynamicDataSource dataSource = new CyDynamicDataSource();
        // 该方法是AbstractRoutingDataSource的方法
        dataSource.setTargetDataSources(targetDataSources);
        // 默认的datasource设置为myTestDbDataSource
        dataSource.setDefaultTargetDataSource(pmiss);
        return dataSource;
    }

    /**
     * 根据数据源创建SqlSessionFactory
     */
    @Bean
    public SqlSessionFactory sqlSessionFactory(@Qualifier("power") DataSource myTestDbDataSource,
                                               @Qualifier("pmiss") DataSource myTestDb2DataSource) throws Exception{
        SqlSessionFactoryBean fb = new SqlSessionFactoryBean();
        fb.setDataSource(this.dataSource(myTestDbDataSource, myTestDb2DataSource));
        fb.setTypeAliasesPackage("org.rcisoft");
        fb.setMapperLocations(new PathMatchingResourcePatternResolver().getResources(localPattern));
        return fb.getObject();
    }



}
