package org.rcisoft.core.aop;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.rcisoft.core.component.CyContextHolder;
import org.rcisoft.core.doc.anno.CyDoc;
import org.rcisoft.core.doc.enums.CyDocEnum;
import org.rcisoft.core.druid.CyDsEnum;

/**
 * 数据源切换
 */
@Aspect
@Slf4j
@CyDoc(CyDocEnum.USELESS)
public class CyDsPowerAspect {

    /**
     *
     * Aopconfig.java  使用
     *
     *     @Bean
     *     public PowerDsAspect powerDsAspect(){
     *         return new PowerDsAspect();
     *     }
     *
     *
     *
     *
     *
     * 切换数据源
     * @param proceedingJoinPoint
     * @return
     * @throws Throwable
     */
    @Around("@annotation(org.rcisoft.core.datasource.anno.CyDsPowerAnno)")
    public Object powerDs(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        try{
            CyContextHolder.setDatabaseType(CyDsEnum.POWER);
            Object obj = proceedingJoinPoint.proceed();
            return obj;
        }catch (Throwable var5){
            log.error(var5.getMessage());
            throw var5;
        }finally {
            CyContextHolder.setDatabaseType(CyDsEnum.PMISS);
        }
    }

}
