package org.rcisoft.core.aop;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.rcisoft.core.doc.anno.CyDoc;
import org.rcisoft.core.doc.enums.CyDocEnum;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.result.CyResultCode;
import org.rcisoft.core.result.CyResultSystemExceptionEnums;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Created with family.
 * author: cy
 * Date: 18/11/1
 * Time: 上午11:00
 * description: redis 操作切面
 */
@Aspect
@Slf4j
@CyDoc(CyDocEnum.USELESS)
public class CyRedisOperateAspect {
    /**
     * 拦截CyRedisServiceImpl 中所有方法,打印相关日志
     * @param proceedingJoinPoint
     *
     */
    @Around("execution (* org.rcisoft.core.service.impl.CyRedisServiceImpl.*(..))")
    public Object redisOperateAspect(ProceedingJoinPoint proceedingJoinPoint) throws InstantiationException, IllegalAccessException {
        String methodName = proceedingJoinPoint.getSignature().getName();
        Type type = null;
        Class className = proceedingJoinPoint.getTarget().getClass();
        //获取类中所有的方法
        Method[] methods = className.getMethods();
        //拦截到方法的参数
        Object[] args = proceedingJoinPoint.getArgs();
        //获取 cyRedisServiceImpl 中对应方法的参数列表
        Class<?>[] paramsCls = this.getParamList(type,methods,methodName,args);
        try {
            //获取方法
            Method method = proceedingJoinPoint.getTarget().getClass()
                    .getMethod(proceedingJoinPoint.getSignature().getName(), paramsCls);
            //获取返回值类型
            type = method.getAnnotatedReturnType().getType();
            //返回值
            Object result = proceedingJoinPoint.proceed();
            //打印日志
            this.dealwithResult(type,result,args,methodName);
            return result;
        } catch (Throwable e) {
            e.printStackTrace();
            throw new CyServiceException(
                    CyResultCode.REDIS_OPERATE_ERROR.getCode()
                    , "redis - "+ methodName+"-error-"+e.getMessage());
        }
    }

    //获取形参方法的参数列表
    private Class<?>[] getParamList(Type type,Method[] methods,String methodName,Object[] args) throws IllegalAccessException, InstantiationException {
        Class<?>[]  paramsCls = new Class<?>[args.length];
        for (int i = 0; i < args.length; ++i) {
            paramsCls[i] = args[i].getClass();
        }
        for(Method m:methods){
            if(m.getName().equals(methodName)){
                //获取返回类型
                Class<?>[] parameterTypes = m.getParameterTypes();
                //当形参为Object时,将实参的参数类型转为Object
                if(parameterTypes.length == args.length){
                    for(int i = 0; i < parameterTypes.length; ++i){
                        if("object".equals(parameterTypes[i].getSimpleName().toLowerCase()))
                            paramsCls[i] = parameterTypes[i];
                        else if("arraylist".equals(paramsCls[i].getSimpleName().toLowerCase()))
                            paramsCls[i] = java.util.List.class;
                    }
                    break;
                }else
                    continue;
            }
        }
        return paramsCls;
    }

    //处理数据
    private void dealwithResult(Type type,Object result,Object[] args,String methodName) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
        String typeName = getParamClssName(type);
        //判断返回值类型是 boolean 的情况
        if("boolean".equals(typeName)){
            if ("true".equals(result))
                log.info("redis " + methodName + " success - " + args[0]);
            else if("false".equals(result))
                log.error("redis " + methodName + " error - " + args[0]);
        }
        //判断返回值类型是 long/double 的情况
        else if(("long".equals(typeName) || "double".equals(typeName))) {
            if (Double.parseDouble(result.toString()) > 0)
                log.info("redis " + methodName + " success - " + args[0]);
            else
                log.error("redis "+ methodName + " failed or key already exists - " + args[0]);
        }
        //判断返回值类型是 list/Map/Object/Set 的情况
        else if("list".equals(typeName)) {
            if(((List)result).size() > 0 )
                log.info("redis " + methodName + " success - " + args[0]);
            else if (((List) result).size() <= 0)
                log.error("redis " + methodName + " failed or get zero info - " + args[0]);
            else if (null != result)
                log.info("redis " + methodName + " success - " + args[0]);
        }else if ("map".equals(typeName)) {
            if (((Map) result).size() > 0)
                log.info("redis " + methodName + " success - " + args[0]);
            else if (((Map) result).size() <= 0)
                log.error("redis " + methodName + " failed or get zero info - " + args[0]);
            else if (null != result)
                log.info("redis " + methodName + " success - " + args[0]);
        } else if ("set".equals(typeName)) {
            if (((Set) result).size() > 0)
                log.info("redis " + methodName + " success - " + args[0]);
            else if (((Set) result).size() <= 0)
                log.error("redis " + methodName + " failed or get zero info - " + args[0]);
            else if (null != result)
                log.info("redis " + methodName + " success - " + args[0]);
        } else if ("object".equals(typeName)) {
            if (null != result)
                log.info("redis " + methodName + " success - " + args[0]);
            else
                log.error("redis " + methodName + " failed or get zero info - " + args[0]);
        }

    }

    //获取String类型的类名  将 java.lang.reflect.Type 转成String
    public String getParamClssName(Type type) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
        if (type instanceof ParameterizedType) {
            return ((Class) ((ParameterizedType) type).getRawType()).getSimpleName().toLowerCase();
        } else if (type instanceof TypeVariable) {
            return ((TypeVariable)type).getGenericDeclaration().toString().toLowerCase();
        } else {
            return ((Class) type).getSimpleName().toLowerCase();
        }
    }
}
