package org.rcisoft.core.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.service.CyAuthService;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by lcy on 17/11/24.
 */
@RestController
@RequestMapping("/auth")
public class CyAuthController {

    @Value("${jwt.header}")
    private String tokenHeader;

    @Autowired
    private CyAuthService authenticationServiceImpl;

    /**
     * 登录
     * @param username
     * @param password
     * @return
     */
    @ApiImplicitParams({@ApiImplicitParam(name = "username", value = "用户名", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "userType", value = "用户类型 (1 admin 2 client)", required = true, dataType = "varchar")})
    @PostMapping(value = "${jwt.route.authentication.path}")
    public CyResult login(@RequestParam("username")String username,
                          @RequestParam("password")String password,
                          @RequestParam("userType")String userType){
        final String token = authenticationServiceImpl.login(username, password, userType);
        CyResult cyResult = CyResultGenUtil.genSuccessResult(token);
        return cyResult;
    }

    /**
     * 注册 根据需要开放注册
     * @param addedUser
     * @return
     */
    //@PostMapping(value = "${jwt.route.authentication.register}")
    public CyResult register(SysUser addedUser){
        authenticationServiceImpl.register(addedUser);
        return CyResultGenUtil.genSuccessResult();
    }

    /**
     * 刷新token 暂不开放
     * @param request
     * @return
     */
    //@GetMapping(value = "${jwt.route.authentication.refresh}")
    public ResponseEntity<?> refreshAndGetAuthenticationToken(
            HttpServletRequest request){
        String token = request.getHeader(tokenHeader);
        String refreshedToken = authenticationServiceImpl.refresh(token);
        if(refreshedToken == null) {
            return ResponseEntity.badRequest().body(null);
        } else {
            return ResponseEntity.ok(refreshedToken);
        }
    }


}
