package org.rcisoft.core.entity;


import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.rcisoft.core.anno.CyOperateFlagAnno;
import org.rcisoft.core.constant.CyDelStatus;
import org.rcisoft.core.constant.CyFlagStatus;

import javax.persistence.MappedSuperclass;
import java.io.Serializable;
import java.util.Date;

/**
 * Created with family.
 * author: cy
 * Date: 16/6/23
 * Time: 上午10:04
 * description: data 实体  常用属性
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@MappedSuperclass
public abstract class CyDataEntity<T>  implements Serializable {
    private static final long serialVersionUID = 1L;

    protected String remarks;	// 备注
    protected String createBy;	// 创建者
    protected Date createDate;// 创建日期

    @CyOperateFlagAnno
    protected String updateBy;	// 更新者

    @CyOperateFlagAnno(jdbcTypeName = "TIMESTAMP")
    protected Date updateDate;// 更新日期

    @CyOperateFlagAnno
    protected String delFlag ; // 删除标记（0：正常；1：删除；2：审核）

    @CyOperateFlagAnno
    protected String flag ; // 启用标记（0：停用；1：启用）

    /**
     * 设置删除
     */
    public void setDeleted(){
        this.setDelFlag(CyDelStatus.DELETED.getStatus());
    }

    /**
     * 设置非删除
     */
    public void setNotDeleted(){
        this.setDelFlag(CyDelStatus.NORMAL.getStatus());
    }


    /**
     * 设置启用
     */
    public void setStart(){
        this.setFlag(CyFlagStatus.NORMAL.getStatus());
    }

    /**
     * 设置停用
     */
    public void setStop(){
        this.setFlag(CyFlagStatus.ABNORMAL.getStatus());
    }

    /**
     * 设置非删除&启用
     */
    public void setNormal(){
        this.setFlag(CyFlagStatus.NORMAL.getStatus());
        this.setDelFlag(CyDelStatus.NORMAL.getStatus());
    }

}
