package org.rcisoft.core.service.impl;


import org.rcisoft.core.util.CyIdGenUtil;
import org.rcisoft.core.util.CyLoggerUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.log.entity.SLog;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.Date;

/**
 * @author guorumeng
 * @version 1.0.0
 * @Description controller层父类，完成日志记录到数据库
 * @date 2018/5/10
 */
@Service
public class CyLogPstServiceImpl {

    /** 数据库记录器 */
    private static final CyLoggerUtil loggerUser = new CyLoggerUtil("DBLog");
    private static final String levelInfo = "0";
    private static final String levelError = "1";

    /**
     * 记录
     * @param
     */
    public void dbInfo(String... mess){
        SLog sLog =  this.initSLog(mess,levelInfo);
        //存入数据库
        loggerUser.info(sLog);
    }

    /**
     * 错误
     * @param
     */
    public void dbError(String... mess){
        SLog sLog =  this.initSLog(mess,levelError);
        //存入数据库
        loggerUser.error(sLog);

    }

    /**
     * 这里可以对SLog填充，在controller中使用时，只需要设置description
     * 因为用户存在session中，所以这里从session中取出用户信息，然后填充到SLog中
     * @param  - 用户操作日志
     * @param
     */
    private SLog initSLog(String[] mess, String logLevel){
        //获取当前会话中的用户和组织机构信息并填充到日志模型实例中
        SLog userLog = new SLog();
        //common//
        userLog.setBusinessId(CyIdGenUtil.uuid());
        if(null != mess && mess.length > 0 )
            userLog.setDetail(mess[0]);
        if(null != mess && mess.length > 1 )
            userLog.setMessage(mess[1]);
        userLog.setLevel(logLevel);
        userLog.setCreateBy(CyUserUtil.getAuthenBusinessId());
        userLog.setNormal();
        userLog.setCreateBy(CyUserUtil.getAuthenBusinessId());
        userLog.setUpdateBy(null);
        Date date = new Date();          // 获取一个Date对象
        userLog.setCreateDate(new Date());
        userLog.setUpdateDate(new Timestamp(date.getTime()));
        return userLog;
    }
}
