package org.rcisoft.core.util;

import io.jsonwebtoken.Claims;
import org.apache.commons.collections.map.HashedMap;
import org.rcisoft.core.bean.CyJwtBean;
import org.rcisoft.core.model.CyJwtUser;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Created by lcy on 17/11/21. *
 * header.payload.signature
 *
 *
 */
@Component
public class CyJwtUtil {

    public CyJwtUtil() {
    }

    private static CyJwtBean cyJwtBean;

    @Autowired
    private CyJwtUtil(CyJwtBean cyJwtBean){
        this.cyJwtBean = cyJwtBean;
    }



    /**
     * 根据token获取用户名
     */
    public static String getUsernameFromToken(String token){
        try {
            Claims claims = cyJwtBean.getClaimsFromToken(token);
            String username = claims.getSubject();
            return username;
        }catch (Exception e){
            return null;
        }
    }

    /**
     * 设置用户信息进jwt
     */
    public static String generateToken(UserDetails userDetails){
        CyJwtUser jwtUser = (CyJwtUser)userDetails;
        Map<String,Object> map = new HashedMap();
        map.put(userDetails.getUsername(),userDetails);
        map.put(CyUserUtil.USER_ID,jwtUser.getBusinessId());
        map.put(CyUserUtil.USER_USERNAME,jwtUser.getUsername());
        return cyJwtBean.generateAccessToken(userDetails.getUsername(),map);
    }

    /**
     * 验证token是否正确
     * @param token
     * @param userDetails
     * @return
     */
    public static boolean validateToken(String token,UserDetails userDetails){
        Claims claims = cyJwtBean.getClaimsFromToken(token);
        LinkedHashMap<String,Object> tokenUserDetails = (LinkedHashMap<String,Object>) claims.get(userDetails.getUsername());
        boolean usernameValid = userDetails.getUsername().equals(tokenUserDetails.get("username"));
        /**
         * usernameValid  理论上 肯定是 true
         */
        if(usernameValid  && !cyJwtBean.isTokenExpired(token) )
            return true;
        else
            return false;
    }
    /*
        获取 user claims
     */
    public static Claims getUserClaimsFromToken(String token){
        try {
            Claims claims = cyJwtBean.getClaimsFromToken(token);
            return claims;
        }catch (Exception e){
            return null;
        }
    }

    /**
     * 刷新 token
     * @param token
     * @return
     */
    public String refreshToken(String token) {
        String refreshedToken;
        try {
            final Claims claims = cyJwtBean.getClaimsFromToken(token);
            refreshedToken = cyJwtBean.generateAccessToken(claims.getSubject(), claims);
        } catch (Exception e) {
            refreshedToken = null;
        }
        return refreshedToken;
    }

    public static CyJwtUser createJwtUser(SysUser user, boolean notLocked){
        return new CyJwtUser(
                user.getBusinessId(),
                user.getLoginName(),
                user.getPassword(),
                notLocked,
                user.getSimpleAuthorities()
        );
    }

}
