package org.rcisoft.core.util;

import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.core.status.ErrorStatus;
import ch.qos.logback.core.status.StatusManager;
import com.mysql.jdbc.StringUtils;
import org.rcisoft.sys.log.entity.SLog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Date;

public class CyLoggerUtil {

	/**
	 * Logback上下文实例
	 */
	private static final LoggerContext context;
	static {
		context = (LoggerContext) LoggerFactory.getILoggerFactory();
	}
	/**
	 * 组合属性
	 * 代理基本日志记录器行为
	 */
	private Logger logger;

	public CyLoggerUtil(Class<?> clazz) {
		logger = LoggerFactory.getLogger(clazz);
	}

	public CyLoggerUtil(String classname) {
		logger = LoggerFactory.getLogger(classname);
	}

	public String getName() {
		return logger.getName();
	}

	public boolean isTraceEnabled() {
		return logger.isTraceEnabled();
	}

	public void trace(String msg) {
		logger.trace(msg);
	}

	public boolean isDebugEnabled() {
		return logger.isDebugEnabled();
	}

	public void debug(String msg) {
		logger.debug(msg);
	}

	public boolean isInfoEnabled() {
		return logger.isInfoEnabled();
	}

	public void info(String msg) {
		logger.info(msg);
	}

	public boolean isWarnEnabled() {
		return logger.isWarnEnabled();
	}

	public void warn(String msg) {
		logger.warn(msg);
	}


	public boolean isErrorEnabled() {
		return logger.isErrorEnabled();
	}

	public void error(String msg) {
		logger.error(msg);
	}

	/**
	 * 打印info信息
	 * @param log
	 */
	public void info(SLog log) {
		this.putMDCproperties(log);
		this.info("");
	}

	/**
	 * 打印error信息
	 * @param log
	 */
	public void error(SLog log) {
		this.putMDCproperties(log);
		this.error("");
	}

	/**
	 * 个性化的，键值对动态组装车间
	 * 反射机制
     * 这个方法主要实现userLog对象中信息解析到logback-spring.xml文件中
	 * @param log
	 */
	private void putMDCproperties(SLog log) {
		try {
			Method[] methods = log.getClass().getMethods();
			//多线程
			MDC.clear();
			for (int i = 0; i < methods.length; ++i) {
				Method method = methods[i];
				if ((!method.getName().startsWith("get"))
						|| (method.getName().equals("getClass"))) {
					continue;
				}
				String pro = method.getName().substring(3, 4).toLowerCase() + method.getName().substring(4);
                Object invoke = method.invoke(log, (Object[]) null);
				String value = null;
				if (invoke instanceof Date) {
					Date date = (Date) invoke;
					value = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);
				} else if(invoke instanceof Object) {
					value = invoke.toString();
				}
				if (!StringUtils.isEmptyOrWhitespaceOnly(pro)){
				    //这个话实现键值对解析到logback-spring.xml文件中
					MDC.put(pro, value);
				}
			}
		} catch (Exception e) {
			StatusManager sm = context.getStatusManager();
			ErrorStatus es = new ErrorStatus("Log属性解析失败！", null, e);
			sm.add(es);
		}
	}

}