package org.rcisoft.sys.log.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.sys.log.entity.SLog;
import org.rcisoft.sys.log.service.SLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
* Created by wsj on 2018-10-25 9:21:45.
*/
@RestController
@RequestMapping("/slog")
@Slf4j
public class SLogController extends CyPaginationController<SLog> {

    @Autowired
    private SLogService sLogServiceImpl;


    @ApiOperation(value="添加日志", notes="添加日志")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public CyResult add(@Valid SLog sLog, BindingResult bindingResult) {
        CyPersistModel data = sLogServiceImpl.save(sLog);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
        sLog);
    }

    @ApiOperation(value="逻辑删除日志", notes="逻辑删除日志")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    @PreAuthorize("hasRole('Add')")
    @Secured({"Add"})
    public CyResult deleteLogical(@PathVariable String businessId,SLog sLog) {
        sLog.setBusinessId(businessId);
        CyPersistModel data = sLogServiceImpl.removeLogical(sLog);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    @ApiOperation(value="删除日志", notes="删除日志")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable String businessId,SLog sLog) {
        sLog.setBusinessId(businessId);
        CyPersistModel data = sLogServiceImpl.remove(sLog);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    @ApiOperation(value="修改日志", notes="修改日志")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable String businessId, @Valid SLog sLog, BindingResult bindingResult) {
        sLog.setBusinessId(businessId);
        CyPersistModel data = sLogServiceImpl.merge(sLog);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
        sLog);
    }

    @ApiOperation(value="查询单一日志", notes="查询单一日志")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable String businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
        sLogServiceImpl.findById(businessId));
    }

    @ApiOperation(value="查询日志表集合", notes="查询日志表集合")
    @GetMapping(value = "/querySLogs")
    public CyResult querySLogs(SLog sLog) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
            sLogServiceImpl.findAll(sLog));
    }

    @ApiOperation(value="分页查询日志集合", notes="分页查询日志集合")
    @GetMapping(value = "/querySLogByPagination")
    public CyGridModel listByPagination(SLog sLog) {
        sLogServiceImpl.findAllByPagination(getPaginationUtility(), sLog);
        return getGridModelResponse();
    }
}
