package org.rcisoft.sys.log.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.aop.CyPageUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.service.CyBaseService;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.log.dao.SLogRepository;
import org.rcisoft.sys.log.entity.SLog;
import org.rcisoft.sys.log.service.SLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * Created by wsj on 2018-10-25 9:21:45.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SLogServiceImpl extends CyBaseService implements SLogService {

    private static final long serialVersionUID = -4923254683034181460L;

    @Autowired
    private SLogRepository sLogRepository;


    /**
    * 保存 日志表
    * @param sLog
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel save(SLog sLog){
        //增加操作
        CyUserUtil.setCurrentPersistOperation(sLog);
        int line = sLogRepository.insertSelective(sLog);
        log.info(CyUserUtil.getUserInfoProp(sLog.getToken(),CyUserUtil.USER_USERNAME)+"新增了ID为"+
            sLog.getBusinessId()+"的日志表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 日志表
    * @param sLog
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SLog sLog){
        int line = sLogRepository.deleteByPrimaryKey(sLog.getBusinessId());
        log.info(CyUserUtil.getUserInfoProp(sLog.getToken(),CyUserUtil.USER_USERNAME)+"删除了ID为"+
            sLog.getBusinessId()+"的日志表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 日志表
    * @param sLog
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SLog sLog){
        CyUserUtil.setCurrentMergeOperation(sLog);
        sLog.setDeleted();
        int line = sLogRepository.logicalDelete(sLog);
        log.info(CyUserUtil.getUserInfoProp(sLog.getToken(),CyUserUtil.USER_USERNAME)+"逻辑删除了ID为"+
            sLog.getBusinessId()+"的日志表信息");
        return new CyPersistModel(line);
    }

    /**
    * 修改 日志表
    * @param sLog
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SLog sLog){
        CyUserUtil.setCurrentMergeOperation(sLog);
        int line = sLogRepository.updateByPrimaryKeySelective(sLog);
        log.info(CyUserUtil.getUserInfoProp(sLog.getToken(),CyUserUtil.USER_USERNAME)+"修改了ID为"+
            sLog.getBusinessId()+"的日志表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 日志表
    * @param id
    * @return
    */
    @Override
    public SLog findById(String id){
        return sLogRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 日志表
    * @param sLog
    * @return
    */
    @Override
    public List<SLog> findAllByPagination(CyPageUtil<SLog> paginationUtility,
        SLog sLog){
        sLog.setStart();
        sLog.setNotDeleted();
        return sLogRepository.querySLogs(sLog);
    }
    /**
    * 查询list 日志表
    * @param sLog
    * @return
    */
    @Override
    public List<SLog> findAll(SLog sLog){
        sLog.setStart();
        sLog.setNotDeleted();
        return sLogRepository.querySLogs(sLog);
    }


}
