package org.rcisoft.sys.role.service.impl;


import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.aop.CyPageUtil;
import org.rcisoft.core.constant.CyDelStatus;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyIdGenUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.dept.entity.DeptRole;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lcy on 16/7/30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysRoleServiceImpl implements SysRoleService {

    @Autowired
    private SysRoleRepository sysRoleRepository;


    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persistRole(SysRole sysRole, String token) {
    	sysRole.setCommonBusinessId();
        //增加操作
        CyUserUtil.setCurrentPersistOperation(sysRole);
        int line = sysRoleRepository.insertSelective(sysRole);
        log.info(CyUserUtil.getUserInfoProp(token,CyUserUtil.USER_USERNAME)+"新增了ID为"+ sysRole.getBusinessId()+"的角色信息");
        return new CyPersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel mergeRole(SysRole sysRole, String token) {
        //修改操作
        CyUserUtil.setCurrentMergeOperation(sysRole);
        int line = sysRoleRepository.updateByPrimaryKeySelective(sysRole);
        log.info(CyUserUtil.getUserInfoProp(token,CyUserUtil.USER_USERNAME)+"修改了ID为"+ sysRole.getBusinessId()+"的角色信息");
        return new CyPersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeRole(String id, String token) {
        //删除该角色所有的信息，包括dept_role  dept_role_user
    	int line;
        line = sysRoleRepository.deleteDeptRoleUserByroleId(id);
        line = sysRoleRepository.deleteDeptRoleByRoleId(id);
        line = sysRoleRepository.deleteRoleMenuByRoleId(id);

        SysRole sysRole = sysRoleRepository.selectByPrimaryKey(id);
        sysRole.setDelFlag(CyDelStatus.DELETED.getStatus());
        CyUserUtil.setCurrentMergeOperation(sysRole);
    	line = sysRoleRepository.updateByPrimaryKeySelective(sysRole);
        log.info(CyUserUtil.getUserInfoProp(token,CyUserUtil.USER_USERNAME)+"删除了ID为"+ sysRole.getBusinessId()+"的角色信息");
        return new CyPersistModel(line);
    }

    @Override
    public List<SysRole> queryRolesByPagination(CyPageUtil<SysRole> paginationUtility, SysRole sysRole) {
        return sysRoleRepository.queryRoles(sysRole);
    }
    @Override
    public List<DeptRole> queryUserRolesByPagination(CyPageUtil<SysRole> paginationUtility, String userId, SysRole sysRole) {
        return sysRoleRepository.queryUserRolesByPagination(userId, sysRole);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel doAuthorization(String roleId, String role_menus, String token) {
        //删除原始权限
        int line = sysRoleRepository.deleteRoleMenus(roleId);
        //批量插入新的权限
        if(null != role_menus && !"".equals(role_menus.trim())){
            String[] menus = role_menus.split(",");
            List<Map> menuRole = new ArrayList<Map>();
            Map map = null;
            for (int i = 0 ; i < menus.length ; i++){
                map = new HashMap();
                map.put("roleId",roleId);
                map.put("menuId",menus[i]);
                map.put("id", CyIdGenUtil.uuid());
                menuRole.add(map);
            }
            line= sysRoleRepository.insertBatchRoleMenu(menuRole);
        }
        log.info(CyUserUtil.getUserInfoProp(token,CyUserUtil.USER_USERNAME)+"更改了ID为"+roleId+"的权限");
        return new CyPersistModel(line);
    }
    
    @Override
    public List<SysRole> queryDeptRolesByPagination(CyPageUtil<SysRole> paginationUtility, String deptId, SysRole sysRole) {
        return sysRoleRepository.queryDeptRolesByPagination(deptId, sysRole);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel doAuthorizationDept(String roleId, String role_depts, String token) {
        //删除原始权限
        int line = sysRoleRepository.deleteRoleDepts(roleId);
        //批量插入新的权限
        if(null != role_depts && !"".equals(role_depts.trim())){
            String[] depts = role_depts.split(",");
            List<Map> deptRole = new ArrayList<Map>();
            Map map = null;
            for (int i = 0 ; i < depts.length ; i++){
                map = new HashMap();
                map.put("roleId",roleId);
                map.put("deptId",depts[i]);
                map.put("id", CyIdGenUtil.uuid());
                deptRole.add(map);
            }
            line = sysRoleRepository.insertBatchRoleDept(deptRole);
        }
        log.info(CyUserUtil.getUserInfoProp(token,CyUserUtil.USER_USERNAME)+"绑定了了ID为"+roleId+"的岗位");
        return new CyPersistModel(line);
    }

	@Override
	public SysRole queryRoleById(String id) {
	       return sysRoleRepository.selectByPrimaryKey(id);
	}

	@Override
	public List<SysRole> queryRolesSelect(SysRole sysRole) {
		return sysRoleRepository.queryRolesSelect(sysRole);
	}




}
